@@ -1,7 +1,7 @@
 // samplv1.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -29,9 +29,11 @@
 #include "samplv1_list.h"
 
 #include "samplv1_fx.h"
-
 #include "samplv1_reverb.h"
 
+#include "samplv1_config.h"
+#include "samplv1_programs.h"
+
 
 #ifdef CONFIG_DEBUG_0
 #include <stdio.h>
@@ -714,7 +716,7 @@ class samplv1_impl
 {
 public:
 
-	samplv1_impl(uint16_t iChannels, uint32_t iSampleRate);
+	samplv1_impl(samplv1 *pSampl, uint16_t iChannels, uint32_t iSampleRate);
 
 	~samplv1_impl();
 
@@ -728,7 +730,12 @@ public:
 	const char *sampleFile() const;
 
 	void setParamPort(samplv1::ParamIndex index, float *pfParam = 0);
-	float *paramPort(samplv1::ParamIndex index);
+	float *paramPort(samplv1::ParamIndex index) const;
+
+	void setParamValue(samplv1::ParamIndex index, float fValue);
+	float paramValue(samplv1::ParamIndex index) const;
+
+	samplv1_programs *programs();
 
 	void process_midi(uint8_t *data, uint32_t size);
 	void process(float **ins, float **outs, uint32_t nframes);
@@ -767,6 +774,9 @@ protected:
 
 private:
 
+	samplv1_config   m_config;
+	samplv1_programs m_programs;
+
 	uint16_t m_iChannels;
 	uint32_t m_iSampleRate;
 
@@ -827,7 +837,9 @@ samplv1_voice::samplv1_voice ( samplv1_impl *pImpl ) :
 
 // engine constructor
 
-samplv1_impl::samplv1_impl ( uint16_t iChannels, uint32_t iSampleRate )
+samplv1_impl::samplv1_impl (
+	samplv1 *pSampl, uint16_t iChannels, uint32_t iSampleRate )
+	: m_programs(pSampl)
 {
 	// null sample.
 	m_gen1.sample0 = 0.0f;
@@ -865,6 +877,9 @@ samplv1_impl::samplv1_impl ( uint16_t iChannels, uint32_t iSampleRate )
 	m_del.bpmsync0 = 0.0f;
 	m_del.bpm0 = 0;
 
+	// load programs database...
+	m_config.loadPrograms(&m_programs);
+
 	// number of channels
 	setChannels(iChannels);
 
@@ -885,6 +900,12 @@ samplv1_impl::samplv1_impl ( uint16_t iChannels, uint32_t iSampleRate )
 
 samplv1_impl::~samplv1_impl (void)
 {
+#if 0
+	// DO NOT save programs database here:
+	// prevent multi-instance clash...
+	m_config.savePrograms(&m_programs);
+#endif
+
 	// deallocate sample filenames
 	setSampleFile(0);
 
@@ -1081,7 +1102,7 @@ void samplv1_impl::setParamPort ( samplv1::ParamIndex index, float *pfParam )
 }
 
 
-float *samplv1_impl::paramPort ( samplv1::ParamIndex index )
+float *samplv1_impl::paramPort ( samplv1::ParamIndex index ) const
 {
 	float *pfParam= 0;
 
@@ -1163,6 +1184,21 @@ float *samplv1_impl::paramPort ( samplv1::ParamIndex index )
 }
 
 
+void samplv1_impl::setParamValue ( samplv1::ParamIndex index, float fValue )
+{
+	float *pfParamPort = paramPort(index);
+	if (pfParamPort)
+		*pfParamPort = fValue;
+}
+
+
+float samplv1_impl::paramValue ( samplv1::ParamIndex index ) const
+{
+	float *pfParamPort = paramPort(index);
+	return (pfParamPort ? *pfParamPort : 0.0f);
+}
+
+
 // handle midi input
 
 void samplv1_impl::process_midi ( uint8_t *data, uint32_t size )
@@ -1184,6 +1220,10 @@ void samplv1_impl::process_midi ( uint8_t *data, uint32_t size )
 	const int status = (data[0] & 0xf0);
 	const int key    = (data[1] & 0x7f);
 
+	// program change
+	if (status == 0xc0)
+		m_programs.prog_change(key);
+	else
 	if (status == 0xd0) {
 		// channel aftertouch
 		m_ctl.pressure = float(key) / 127.0f;
@@ -1284,7 +1324,11 @@ void samplv1_impl::process_midi ( uint8_t *data, uint32_t size )
 	}
 	// control change
 	else if (status == 0xb0) {
-		switch (key) {
+	switch (key) {
+		case 0x00:
+			// bank-select MSB (cc#0)
+			m_programs.bank_select_msb(value);
+			break;
 		case 0x01:
 			// modulation wheel (cc#1)
 			m_ctl.modwheel = *m_def.modwheel * float(value) / 127.0f;
@@ -1297,6 +1341,10 @@ void samplv1_impl::process_midi ( uint8_t *data, uint32_t size )
 			// channel panning (cc#10)
 			m_ctl.panning = float(value - 64) / 64.0f;
 			break;
+		case 0x20:
+			// bank-select LSB (cc#32)
+			m_programs.bank_select_lsb(value);
+			break;
 		case 0x40:
 			// sustain/damper pedal (cc#64)
 			if (m_ctl.sustain && value <  64)
@@ -1435,6 +1483,14 @@ void samplv1_impl::reset (void)
 }
 
 
+// programs accessor
+
+samplv1_programs *samplv1_impl::programs (void)
+{
+	return &m_programs;
+}
+
+
 // synthesize
 
 void samplv1_impl::process ( float **ins, float **outs, uint32_t nframes )
@@ -1654,13 +1710,14 @@ void samplv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 }
 
 
+
 //-------------------------------------------------------------------------
 // samplv1 - decl.
 //
 
 samplv1::samplv1 ( uint16_t iChannels, uint32_t iSampleRate )
 {
-	m_pImpl = new samplv1_impl(iChannels, iSampleRate);
+	m_pImpl = new samplv1_impl(this, iChannels, iSampleRate);
 }
 
 
@@ -1761,6 +1818,17 @@ float *samplv1::paramPort ( ParamIndex index ) const
 }
 
 
+void samplv1::setParamValue ( ParamIndex index, float fValue )
+{
+	m_pImpl->setParamValue(index, fValue);
+}
+
+float samplv1::paramValue ( ParamIndex index ) const
+{
+	return m_pImpl->paramValue(index);
+}
+
+
 void samplv1::process_midi ( uint8_t *data, uint32_t size )
 {
 #ifdef CONFIG_DEBUG_0
@@ -1780,6 +1848,14 @@ void samplv1::process ( float **ins, float **outs, uint32_t nframes )
 }
 
 
+// programs accessor
+
+samplv1_programs *samplv1::programs (void) const
+{
+	return m_pImpl->programs();
+}
+
+
 // all reset clear
 
 void samplv1::reset (void)
