@@ -1,7 +1,7 @@
 // samplv1_lv2.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -65,7 +65,7 @@ samplv1_lv2::samplv1_lv2 (
 		}
 	}
 
-	const uint16_t nchannels = channels();
+	const uint16_t nchannels = samplv1::channels();
 	m_ins  = new float * [nchannels];
 	m_outs = new float * [nchannels];
 	for (uint16_t k = 0; k < nchannels; ++k)
@@ -99,7 +99,7 @@ void samplv1_lv2::connect_port ( uint32_t port, void *data )
 		m_outs[1] = (float *) data;
 		break;
 	default:
-		setParamPort(ParamIndex(port - ParamBase), (float *) data);
+		samplv1::setParamPort(samplv1::ParamIndex(port - ParamBase), (float *) data);
 		break;
 	}
 }
@@ -107,7 +107,7 @@ void samplv1_lv2::connect_port ( uint32_t port, void *data )
 
 void samplv1_lv2::run ( uint32_t nframes )
 {
-	const uint16_t nchannels = channels();
+	const uint16_t nchannels = samplv1::channels();
 	float *ins[nchannels], *outs[nchannels];
 	for (uint16_t k = 0; k < nchannels; ++k) {
 		ins[k]  = m_ins[k];
@@ -124,14 +124,14 @@ void samplv1_lv2::run ( uint32_t nframes )
 				uint8_t *data = (uint8_t *) LV2_ATOM_BODY(&event->body);
 				const uint32_t nread = event->time.frames - ndelta;
 				if (nread > 0) {
-					process(ins, outs, nread);
+					samplv1::process(ins, outs, nread);
 					for (uint16_t k = 0; k < nchannels; ++k) {
 						ins[k]  += nread;
 						outs[k] += nread;
 					}
 				}
 				ndelta = event->time.frames;
-				process_midi(data, event->body.size);
+				samplv1::process_midi(data, event->body.size);
 			}
 			else
 			if (event->body.type == m_urids.atom_Blank ||
@@ -143,13 +143,13 @@ void samplv1_lv2::run ( uint32_t nframes )
 					lv2_atom_object_get(object,
 						m_urids.time_beatsPerMinute, &atom, NULL);
 					if (atom && atom->type == m_urids.atom_Float) {
-						const float *bpmsync = paramPort(samplv1::DEL1_BPMSYNC);
-						if (bpmsync && *bpmsync > 0.0f) {
-							float *bpmhost = paramPort(samplv1::DEL1_BPMHOST);
-							if (bpmhost) {
+						const float bpm_sync = samplv1::paramValue(samplv1::DEL1_BPMSYNC);
+						if (bpm_sync > 0.0f) {
+							const float bpm_host = samplv1::paramValue(samplv1::DEL1_BPMHOST);
+							if (bpm_host > 0.0f) {
 								const float bpm	= ((LV2_Atom_Float *) atom)->body;
-								if (::fabs(*bpmhost - bpm) > 0.01f)
-									*bpmhost = bpm;
+								if (::fabs(bpm_host - bpm) > 0.01f)
+									samplv1::setParamValue(samplv1::DEL1_BPMHOST, bpm);
 							}
 						}
 					}
@@ -159,19 +159,19 @@ void samplv1_lv2::run ( uint32_t nframes )
 	//	m_atom_sequence = NULL;
 	}
 
-	process(ins, outs, nframes - ndelta);
+	samplv1::process(ins, outs, nframes - ndelta);
 }
 
 
 void samplv1_lv2::activate (void)
 {
-	reset();
+	samplv1::reset();
 }
 
 
 void samplv1_lv2::deactivate (void)
 {
-	reset();
+	samplv1::reset();
 }
 
 
@@ -225,7 +225,8 @@ static LV2_State_Status samplv1_lv2_state_save ( LV2_Handle instance,
 
 	size_t size = ::strlen(value) + 1;
 
-	LV2_State_Status result = (*store)(handle, key, value, size, type, flags);
+	const LV2_State_Status result
+		= (*store)(handle, key, value, size, type, flags);
 
 	if (map_path)
 		::free((void *) value);
@@ -335,7 +336,7 @@ static LV2_State_Status samplv1_lv2_state_restore ( LV2_Handle instance,
 	if (loop_start < loop_end)
 		pPlugin->setLoopRange(loop_start, loop_end);
 
-	samplv1_sched::sync_notify();
+	samplv1_sched::sync_notify(samplv1_sched::Sample);
 
 	return LV2_STATE_SUCCESS;
 }
@@ -348,6 +349,44 @@ static const LV2_State_Interface samplv1_lv2_state_interface =
 };
 
 
+#ifdef CONFIG_LV2_PROGRAMS
+
+#include "samplv1_programs.h"
+
+const LV2_Program_Descriptor *samplv1_lv2::get_program ( uint32_t index )
+{
+	samplv1_programs *pPrograms = samplv1::programs();
+	const samplv1_programs::Banks& banks = pPrograms->banks();
+	samplv1_programs::Banks::ConstIterator bank_iter = banks.constBegin();
+	const samplv1_programs::Banks::ConstIterator& bank_end = banks.constEnd();
+	for (uint32_t i = 0; bank_iter != bank_end; ++bank_iter) {
+		samplv1_programs::Bank *pBank = bank_iter.value();
+		const samplv1_programs::Progs& progs = pBank->progs();
+		samplv1_programs::Progs::ConstIterator prog_iter = progs.constBegin();
+		const samplv1_programs::Progs::ConstIterator& prog_end = progs.constEnd();
+		for ( ; prog_iter != prog_end; ++prog_iter, ++i) {
+			samplv1_programs::Prog *pProg = prog_iter.value();
+			if (i >= index) {
+				m_aProgramName = pProg->name().toUtf8();
+				m_program.bank = pBank->id();
+				m_program.program = pProg->id();
+				m_program.name = m_aProgramName.constData();
+				return &m_program;
+			}
+		}
+	}
+
+	return NULL;
+}
+
+void samplv1_lv2::select_program ( uint32_t bank, uint32_t program )
+{
+	samplv1::programs()->select_program(bank, program);
+}
+
+#endif	// CONFIG_LV2_PROGRAMS
+
+
 //-------------------------------------------------------------------------
 // samplv1_lv2 - LV2 desc.
 //
@@ -401,8 +440,41 @@ static void samplv1_lv2_cleanup ( LV2_Handle instance )
 }
 
 
+#ifdef CONFIG_LV2_PROGRAMS
+
+static const LV2_Program_Descriptor *samplv1_lv2_programs_get_program (
+	LV2_Handle instance, uint32_t index )
+{
+	samplv1_lv2 *pPlugin = static_cast<samplv1_lv2 *> (instance);
+	if (pPlugin)
+		return pPlugin->get_program(index);
+	else
+		return NULL;
+}
+
+static void samplv1_lv2_programs_select_program (
+	LV2_Handle instance, uint32_t bank, uint32_t program )
+{
+	samplv1_lv2 *pPlugin = static_cast<samplv1_lv2 *> (instance);
+	if (pPlugin)
+		pPlugin->select_program(bank, program);
+}
+
+static const LV2_Programs_Interface samplv1_lv2_programs_interface =
+{
+	samplv1_lv2_programs_get_program,
+	samplv1_lv2_programs_select_program,
+};
+
+#endif	// CONFIG_LV2_PROGRAMS
+
 static const void *samplv1_lv2_extension_data ( const char *uri )
 {
+#ifdef CONFIG_LV2_PROGRAMS
+	if (::strcmp(uri, LV2_PROGRAMS__Interface) == 0)
+		return (void *) &samplv1_lv2_programs_interface;
+	else
+#endif
 	if (::strcmp(uri, LV2_STATE__interface))
 		return NULL;
 
