@@ -1,7 +1,7 @@
 // samplv1_param.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -19,8 +19,8 @@
 
 *****************************************************************************/
 
-#include "samplv1_config.h"
 #include "samplv1_param.h"
+#include "samplv1_config.h"
 
 #include <QHash>
 
@@ -168,6 +168,7 @@ void samplv1_param::loadSamples (
 			if (sFilename.isEmpty())
 				sFilename = eSample.text();
 			// Done it.
+			sFilename = QFileInfo(sFilename).canonicalFilePath();
 			pSampl->setSampleFile(sFilename.toUtf8().constData());
 			// Set actual sample loop points...
 			pSampl->setLoopRange(iLoopStart, iLoopEnd);
@@ -219,7 +220,21 @@ void samplv1_param::loadPreset ( samplv1 *pSampl, const QString& sFilename )
 	if (pSampl == NULL)
 		return;
 
-	QFile file(sFilename);
+	QFileInfo fi(sFilename);
+	if (!fi.exists()) {
+		samplv1_config *pConfig = samplv1_config::getInstance();
+		if (pConfig) {
+			const QString& sPresetFile
+				= pConfig->presetFile(sFilename);
+			if (sPresetFile.isEmpty())
+				return;
+			fi.setFile(sPresetFile);
+			if (!fi.exists())
+				return;
+		}
+	}
+
+	QFile file(fi.filePath());
 	if (!file.open(QIODevice::ReadOnly))
 		return;
 
@@ -231,7 +246,6 @@ void samplv1_param::loadPreset ( samplv1 *pSampl, const QString& sFilename )
 		}
 	}
 
-	const QFileInfo fi(sFilename);
 	const QDir currentDir(QDir::current());
 	QDir::setCurrent(fi.absolutePath());
 
@@ -266,14 +280,12 @@ void samplv1_param::loadPreset ( samplv1 *pSampl, const QString& sFilename )
 									continue;
 								index = s_hash.value(sName);
 							}
-							float fParamValue = eParam.text().toFloat();
+							float fValue = eParam.text().toFloat();
 						#if 1//--legacy support < 0.3.0.4
-							if (index == samplv1::DEL1_BPM && fParamValue < 3.6f)
-								fParamValue *= 100.0f;
+							if (index == samplv1::DEL1_BPM && fValue < 3.6f)
+								fValue *= 100.0f;
 						#endif
-							float *pfParamPort = pSampl->paramPort(index);
-							if (pfParamPort)
-								*pfParamPort = fParamValue;
+							pSampl->setParamValue(index, fValue);
 						}
 					}
 				}
@@ -294,15 +306,13 @@ void samplv1_param::savePreset ( samplv1 *pSampl, const QString& sFilename )
 	if (pSampl == NULL)
 		return;
 
-	const QString& sPreset = QFileInfo(sFilename).completeBaseName();
-
 	const QFileInfo fi(sFilename);
 	const QDir currentDir(QDir::current());
 	QDir::setCurrent(fi.absolutePath());
 
 	QDomDocument doc(SAMPLV1_TITLE);
 	QDomElement ePreset = doc.createElement("preset");
-	ePreset.setAttribute("name", sPreset);
+	ePreset.setAttribute("name", fi.completeBaseName());
 	ePreset.setAttribute("version", SAMPLV1_VERSION);
 
 	QDomElement eSamples = doc.createElement("samples");
@@ -315,18 +325,14 @@ void samplv1_param::savePreset ( samplv1 *pSampl, const QString& sFilename )
 		samplv1::ParamIndex index = samplv1::ParamIndex(i);
 		eParam.setAttribute("index", QString::number(i));
 		eParam.setAttribute("name", samplv1_param::paramName(index));
-		const float *pfParamPort = pSampl->paramPort(index);
-		float fParamValue = 0.0f;
-		if (pfParamPort)
-			fParamValue = *pfParamPort;
-		eParam.appendChild(
-			doc.createTextNode(QString::number(fParamValue)));
+		const float fValue = pSampl->paramValue(index);
+		eParam.appendChild(doc.createTextNode(QString::number(fValue)));
 		eParams.appendChild(eParam);
 	}
 	ePreset.appendChild(eParams);
 	doc.appendChild(ePreset);
 
-	QFile file(sFilename);
+	QFile file(fi.filePath());
 	if (file.open(QIODevice::WriteOnly | QIODevice::Truncate)) {
 		QTextStream(&file) << doc.toString();
 		file.close();
