@@ -27,7 +27,8 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <sndfile.h>
+// forward decls.
+class samplv1_reverse_sched;
 
 
 //-------------------------------------------------------------------------
@@ -39,15 +40,10 @@ class samplv1_sample
 public:
 
 	// ctor.
-	samplv1_sample(float srate = 44100.0f)
-		: m_srate(srate), m_filename(0), m_nchannels(0),
-			m_rate0(0.0f), m_freq0(1.0f), m_ratio(0.0f),
-			m_nframes(0), m_pframes(0), m_loop(false),
-			m_loop_start(0), m_loop_end(0) {}
+	samplv1_sample(float srate = 44100.0f);
 
 	// dtor.
-	~samplv1_sample()
-		{ close(); }
+	~samplv1_sample();
 
 	// nominal sample-rate.
 	void setSampleRate(float srate)
@@ -55,8 +51,32 @@ public:
 	float sampleRate() const
 		{ return m_srate; }
 
+	// reverse mode.
+	void setReverse(bool reverse)
+	{
+		if (( m_reverse && !reverse) ||
+			(!m_reverse &&  reverse)) {
+			m_reverse = reverse;
+			reverse_sample();
+		}
+	}
+
+	bool isReverse() const
+		{ return m_reverse; }
+
+	// schedule sample reverse.
+	void reverse_test(bool reverse)
+	{
+		if (( m_reverse && !reverse) ||
+			(!m_reverse &&  reverse)) {
+			reverse_sched(reverse);
+		}
+	}
+
+	void reverse_sched(bool reverse);
+
 	// loop mode.
-	void setLoop ( bool loop )
+	void setLoop(bool loop)
 	{
 		m_loop = loop;
 
@@ -69,8 +89,17 @@ public:
 	bool isLoop() const
 		{ return m_loop && (m_loop_start < m_loop_end); }
 
+	// loop change.
+	void loop_test(bool loop)
+	{
+		if (( m_loop && !loop) ||
+			(!m_loop &&  loop)) {
+			setLoop(loop);
+		}
+	}
+
 	// loop range.
-	void setLoopRange ( uint32_t start, uint32_t end )
+	void setLoopRange(uint32_t start, uint32_t end)
 	{
 		if (start > m_nframes)
 			start = m_nframes;
@@ -93,77 +122,8 @@ public:
 		{ return m_loop_end; }
 
 	// init.
-	bool open(const char *filename, float freq0 = 1.0f)
-	{
-		if (!filename)
-			return false;
-
-		close();
-
-		m_filename = ::strdup(filename);
-
-		SF_INFO info;
-		::memset(&info, 0, sizeof(info));
-
-		SNDFILE *file = ::sf_open(m_filename, SFM_READ, &info);
-		if (!file)
-			return false;
-
-		m_nchannels = info.channels;
-		m_rate0     = float(info.samplerate);
-		m_nframes   = info.frames;
-
-		const uint32_t nsize = m_nframes + 4;
-
-		m_pframes = new float * [m_nchannels];
-		for (uint16_t k = 0; k < m_nchannels; ++k) {
-			m_pframes[k] = new float [nsize];
-			::memset(m_pframes[k], 0, nsize * sizeof(float));
-		}
-
-		float *buffer = new float [m_nchannels * m_nframes];
-
-		const int nread = ::sf_readf_float(file, buffer, m_nframes);
-		if (nread > 0) {
-			const uint32_t n = uint32_t(nread);
-			uint32_t i = 0;
-			for (uint32_t j = 0; j < n; ++j) {
-				for (uint16_t k = 0; k < m_nchannels; ++k)
-					m_pframes[k][j] = buffer[i++];
-			}
-		}
-
-		delete [] buffer;
-		::sf_close(file);
-
-		reset(freq0);
-
-		setLoop(m_loop);
-		return true;
-	}
-
-	void close()
-	{
-		if (m_pframes) {
-			for (uint16_t k = 0; k < m_nchannels; ++k)
-				delete [] m_pframes[k];
-			delete [] m_pframes;
-			m_pframes = 0;
-		}
-
-		m_nframes   = 0;
-		m_ratio     = 0.0f;
-		m_freq0     = 1.0f;
-		m_rate0     = 0.0f;
-		m_nchannels = 0;
-
-		if (m_filename) {
-			::free(m_filename);
-			m_filename = 0;
-		}
-
-		setLoopRange(0, 0);
-	}
+	bool open(const char *filename, float freq0 = 1.0f);
+	void close();
 
 	// accessors.
 	const char *filename() const
@@ -196,6 +156,11 @@ public:
 	bool isOver(uint32_t frame) const
 		{ return !m_pframes || (frame >= m_nframes); }
 
+protected:
+
+	// reverse sample buffer.
+	void reverse_sample();
+
 private:
 
 	float    m_srate;
@@ -206,9 +171,12 @@ private:
 	float    m_ratio;
 	uint32_t m_nframes;
 	float  **m_pframes;
+	bool     m_reverse;
 	bool     m_loop;
 	float    m_loop_start;
 	float    m_loop_end;
+
+	samplv1_reverse_sched *m_reverse_sched;
 };
 
 
