@@ -598,11 +598,11 @@ void samplv1widget::updateParamEx ( samplv1::ParamIndex index, float fValue )
 	++m_iUpdate;
 
 	switch (index) {
-#if 0//--updateSchedNotify(samplv1_sched::Sample);
+#if 1//--updateSchedNotify(samplv1_sched::Sample);
 	case samplv1::GEN1_REVERSE: {
 		const bool bReverse = bool(fValue > 0.0f);
-		pSampl->setReverse(bReverse);
-		updateSample(pSampl->sample());
+		pSamplUi->setReverse(bReverse);
+		updateSample(pSamplUi->sample());
 		break;
 	}
 #endif
