@@ -1,7 +1,7 @@
 // samplv1widget.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2015, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -24,9 +24,9 @@
 
 #include "ui_samplv1widget.h"
 
-#include "samplv1widget_config.h"
+#include "samplv1_config.h"
 
-#include "samplv1.h"
+#include "samplv1_ui.h"
 
 
 // forward decls.
@@ -92,9 +92,6 @@ protected slots:
 	// Sample loader slot.
 	void loadSample(const QString& sFilename);
 
-	// Sample updater.
-	void updateSampleNotify();
-
 	// Common context menu.
 	void contextMenuRequest(const QPoint& pos);
 
@@ -112,8 +109,11 @@ protected slots:
 	// Delay BPM change.
 	void bpmSyncChanged();
 
+	// Notification updater.
+	void updateSchedNotify(int stype);
+
 	// Menu actions.
-	void helpUseNativeDialogs(bool bOn);
+	void helpConfigure();
 
 	void helpAbout();
 	void helpAboutQt();
@@ -121,7 +121,7 @@ protected slots:
 protected:
 
 	// Synth engine accessor.
-	virtual samplv1 *instance() const = 0;
+	virtual samplv1_ui *ui_instance() const = 0;
 
 	// Reset swap params A/B group.
 	void resetSwapParams();
@@ -151,6 +151,9 @@ protected:
 	// Update loop range change status.
 	void updateSampleLoop(samplv1_sample *pSample, bool bDirty = false);
 
+	// Preset status updater.
+	void updateLoadPreset(const QString& sPreset);
+
 	// Dirty flag (overridable virtual) methods.
 	virtual void updateDirtyPreset(bool bDirtyPreset);
 
@@ -159,8 +162,6 @@ private:
 	// Instance variables.
 	Ui::samplv1widget m_ui;
 
-	samplv1widget_config m_config;
-
 	samplv1_sched_notifier *m_sched_notifier;
 
 	QHash<samplv1::ParamIndex, samplv1widget_knob *> m_paramKnobs;
